import Util from '../lib/Util';

class Translator {
    constructor() {
        this.langs = [];
        this.cache = {};
        this.provider = null;
    }

    static addProvider(alias, Factory) {
        Translator.providerMap[alias] = Factory;
    }

    static createProvider(alias, key, uri) {
        if (typeof Translator.providerMap[alias] === 'undefined') {
            return null;
        }

        const Factory = Translator.providerMap[alias];

        return new Factory(key, uri);
    }

    setup(alias, key, uri) {
        this.provider = Translator.createProvider(alias, key, uri);

        return this;
    }

    getLangs() {
        if (this.langs.length) {
            return Promise.resolve(this.langs);
        }

        return this.provider.getLangs();
    }

    translate(text, to, from) {
        const hash = Util.composeHash(text);
        const lang = Util.composeLangsAlias(from, to);

        if (typeof this.cache[lang] !== 'undefined' && typeof this.cache[lang][hash] !== 'undefined') {
            return Promise.resolve(this.cache[lang][hash]);
        }

        return this.provider.translate(text, to, from);
    }

    lookup(text, to, from) {
        const lang = Util.composeLangsAlias(from, to);

        return this.getLangs()
            .then(langs => {
                this.langs = langs;

                if (!this.provider.supports(langs, to, from)) {
                    return null;
                }

                return this.translate(text, to, from);
            })
            .then(trans => {
                if (typeof this.cache[lang] === 'undefined') {
                    this.cache[lang] = {};
                }
                this.cache[lang][Util.composeHash(text)] = trans;

                return trans;
            })
        ;
    }
}

Translator.providerMap = {};

export default Translator;
